# The name of the kmod package here.
%define kmod_name kpatch
%define module %{kmod_name}.ko

%define kernel_version 3.10.0-862.20.2.vz7.73.24
%define vzver %(echo '%{kernel_version}' | sed -e 's/^.*vz7\\.//')

# These defines are convenient to make the spec files for VZ7 and VL7 as
# close as possible.
%define kversion_base %{kernel_version}
%define kversion_dist %{kversion_base}
%define kversion %{kversion_dist}

%global _use_internal_dependency_generator 0

# Disable the automatic building of the debug package(s).
%define debug_package %{nil}

# Do not strip files automatically, this would strip the module's signature
# as well
%define __strip /usr/bin/true

# The suffix of the subpackage's name.
%define subpkg %{vzver}

# The major release number. Increase it when rebuilding the package.
%define rel 1


Name:		%{kmod_name}-kmod
Version:	0.5.0
Release:	%{rel}.%{vzver}%{?dist}
# Epoch should be higher here than in the dist-vz7-uN branch the autobuilder
# uses. This makes sure the packages built from version-specific branches
# are preferred over those built automatically. No need to track the release
# numbers of the latter ones, etc.
Epoch:		1

Summary:	Dynamic kernel patching tools - the kernel-mode part

Group:		System Environment/Kernel
License:	GPLv2
URL:		https://github.com/dynup/kpatch

Source0:	https://github.com/dynup/kpatch/archive/v%{version}.tar.gz?/%{kmod_name}-%{version}.tar.gz

# Note. The patchset is the same as for kpatch packages. Some of the patches
# have nothing to do with the core kernel module but it is easier to maintain
# the packaging projects for kpatch and kpatch-kmod this way.

# The patches from upstream.
Patch1:		0001-updated-man-page.patch
Patch2:		0002-version-update.patch
Patch3:		0003-dnf-to-yum.patch
Patch4:		0004-fixing-RHEL-and-oracle-typos-as-well.patch
Patch5:		0005-kpatch-set-LC_ALL-C-explicitly-when-loading-the-patc.patch
Patch6:		0006-kpatch-make-it-easier-to-tune-the-number-of-attempts.patch
Patch7:		0007-kmod-core-fix-definition-of-KERNELRELEASE.patch
Patch8:		0008-Do-not-mention-initrd-in-the-help-messages-and-in-th.patch
Patch9:		0009-kpatch-build-hint-is-not-needed-in-kpatch_create_-_s.patch
Patch10:	0010-If-kpatch-fails-to-disable-a-patch-retry-a-few-times.patch
Patch11:	0011-kmod-core-fix-compilation-with-CONFIG_HAVE_ARCH_PREL.patch
Patch12:	0012-kmod-patch-fix-patch-linking-with-4.20.patch

# Virtuozzo-specific patches.
Patch501:	vz001-kpatch-build-Support-Virtuozzo-Linux-as-well.patch
Patch502:	vz002-kpatch-increase-the-number-of-load-attempts-and-the-.patch
Patch503:	vz003-kmod-core-Do-not-taint-the-kernel.patch
Patch504:	vz004-kmod-core-added-a-minimal-fault-simulation-support.patch
Patch505:	vz005-Support-both-the-older-and-the-newer-ABI-of-the-core.patch
Patch506:	vz006-Revert-kpatch-deprecate-the-replace-command.patch
Patch507:	vz007-Added-logging-to-var-log-kpatch-v2.patch
Patch508:	vz008-Keep-only-the-basic-info-in-the-README.patch
Patch509:	vz009-kpatch-build-x86-do-not-use-the-patched-functions-as.patch
Patch510:	vz010-kpatch-build-use-prefix-kpatch-for-livepatch-modules.patch
Patch511:	vz011-Support-atomic-replace-in-the-livepatch-modules.patch
Patch512:	vz012-Use-atomic-replace-only-when-upgrading-official-cumu.patch
Patch513:	vz013-kmod-add-support-for-in-kernel-livepatch-hooks.patch
Patch514:	vz014-kmod-patch-Use-the-newer-callbacks-starting-from-ker.patch
Patch515:	vz015-kpatch-unload-disabled-patch-modules-before-atomic-r.patch
Patch516:	vz016-kpatch-fix-for-module-in-SYSFS-loops.patch
Patch517:	vz017-kpatch-kpatch-list-should-not-show-patch-state-by-de.patch

# https://github.com/dynup/kpatch/issues/783
Patch518:	vz018-kpatch-build-don-t-die-if-undefined-symbols-are-foun.patch

Patch519:	vz019-Wait-for-pending-RCU-callbacks-to-complete-before-ap.patch
Patch520:	vz020-core-allow-freeing-all-shadow-variables-with-the-giv.patch
Patch521:	vz021-kpatch-build-allow-making-Kpatch-compatible-patches-.patch
Patch522:	vz022-Make-sure-kpatch-core-module-is-used-if-available.patch
Patch523:	vz023-kmod-core-add-a-fake-parameter-enable.patch
Patch524:	vz024-PPC64le-do-not-use-the-patched-functions-as-callback.patch
Patch525:	vz025-create-diff-object-skip-dynrelas-for-function-pointe.patch

ExclusiveArch:	x86_64

BuildRequires:	vzkernel-devel = %{kernel_version}
BuildRequires:	perl
BuildRequires:	openssl
BuildRequires:	module-signing-keys
BuildRequires:	elfutils >= 0.153-1

%description
This package contains %{module} kernel module, the kernel-mode part of
Kpatch. Kpatch is a Linux dynamic kernel patching tool which allows you
to patch a running kernel without rebooting or restarting any processes.

#===========================================================================

%package %{subpkg}

# The name of the package already includes vzversion, no need to
# keep it in the Release expression.
Release:	%{rel}%{?dist}
Summary:	Dynamic kernel patch manager - the kernel-mode part
Group:		System Environment/Kernel
License:	GPLv2

# Do not generate requirements automatically. By default the requirements
# like "kernel(__kmalloc) = abcd1234" are generated and tie the package to
# the corresponding kernel versions.
# kpatch-patch should pull the right kpatch-kmod package, so it is not
# needed to require the particular kernel explicitly. Such requirements
# proved to be problematic.
AutoReq: no

Provides:	kernel-modules >= %{kversion_dist}.%{_target_cpu}
# kpatch-build requires kpatch-kmod >= <...>
Provides:	%{kmod_name}-kmod = %{?epoch:%{epoch}:}%{version}-%{release}
# Just in case someone looks for kmod-kpatch-<...>:
Provides:	kmod-%{kmod_name}-%{kversion_base} = %{?epoch:%{epoch}:}%{version}-%{release}
# kpatch-patch requires kpatch-kmod-<kernel_version>
Provides:	%{kmod_name}-kmod-%{kversion_base} = %{?epoch:%{epoch}:}%{version}-%{release}

Requires(post):		/usr/sbin/depmod
Requires(post):		kmod
Requires(postun):	/usr/sbin/depmod
Requires(postun):	kmod

# The plugin will automatically remove unneeded kpatch-kmod* and
# readykernel-patch* packages.
Requires:	yum-plugin-readykernel

%description %{subpkg}
This package contains %{module} kernel module, the kernel-mode part of
Kpatch. Kpatch is a Linux dynamic kernel patching tool which allows you
to patch a running kernel without rebooting or restarting any processes.


%files %{subpkg}
%defattr(644,root,root,755)
%dir /lib/modules/%{kversion}
%dir /lib/modules/%{kversion}/extra
%dir /lib/modules/%{kversion}/extra/%{kmod_name}
/lib/modules/%{kversion}/extra/%{kmod_name}/*.ko
/lib/modules/%{kversion}/extra/%{kmod_name}/*.symvers


%post %{subpkg}
echo "Running depmod..."
if [ -e "/boot/System.map-%{kversion}" ]; then
    /usr/sbin/depmod -aeF "/boot/System.map-%{kversion}" "%{kversion}" > /dev/null || :
fi
echo "Done."


%postun %{subpkg}
echo "Running depmod..."
if [ -e "/boot/System.map-%{kversion}" ]; then
    /usr/sbin/depmod -aeF "/boot/System.map-%{kversion}" "%{kversion}" > /dev/null || :
fi
echo "Done."

#===========================================================================

%package %{subpkg}-debuginfo
Summary:	Debug info for %{name}-%{subpkg}

%description %{subpkg}-debuginfo
Debug info for %{name}-%{subpkg}.

%files %{subpkg}-debuginfo
/lib/modules/%{kversion}/extra/%{kmod_name}/*.debug

#===========================================================================

%prep
%setup -qn %{kmod_name}-%{version}

# apply the patches from upstream first
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
%patch7 -p1
%patch8 -p1
%patch9 -p1
%patch10 -p1
%patch11 -p1
%patch12 -p1

# ...then apply VZ-specific patches
%patch501 -p1
%patch502 -p1
%patch503 -p1
%patch504 -p1
%patch505 -p1
%patch506 -p1
%patch507 -p1
%patch508 -p1
%patch509 -p1
%patch510 -p1
%patch511 -p1
%patch512 -p1
%patch513 -p1
%patch514 -p1
%patch515 -p1
%patch516 -p1
%patch517 -p1
%patch518 -p1
%patch519 -p1
%patch520 -p1
%patch521 -p1
%patch522 -p1
%patch523 -p1
%patch524 -p1
%patch525 -p1


%build
echo "Building Kpatch core module for the kernel %{kversion}."
echo "Using the default Kpatch core ABI."
make KPATCH_BUILD=/usr/src/kernels/%{kversion}/ -C kmod/core %{_smp_mflags}


%install
rm -rf %{buildroot}

install -d %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/
install -m 644 kmod/core/%{module} %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/
install -m 644 kmod/core/Module.symvers %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/

# Strip debug info from the module.
# Do not use _{__strip} macro here because it is redefined above.
pushd %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/
eu-strip --remove-comment --reloc-debug-sections -f %{kmod_name}.debug %{module}
popd

# Sign the module if the keys are available.
KEYPATH=%{_datadir}/module-signing-keys/kpatch/
MODSECKEY="${KEYPATH}/signing_key_kpatch.priv"
MODPUBKEY="${KEYPATH}/signing_key_kpatch.x509"
MODULE_FILE="%{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/%{module}"

if [[ -f "${MODSECKEY}" ]]; then
	/usr/src/kernels/%{kversion}/scripts/sign-file sha256 \
	  ${MODSECKEY} ${MODPUBKEY} ${MODULE_FILE}
	rm -f ${MODULE_FILE}.{sig,dig}

	if [ "~Module signature appended~" != "$(tail -c 28 ${MODULE_FILE})" ]; then
		echo "Module %{module} is still unsigned."
		exit 1
	fi
fi
